@@ -77,7 +77,6 @@ impl Approximate<types::Item> for Query {
             sims.append(&mut kind.approx(&item.inner, generics, substs))
         }
 
-        trace!("sims: {:?}", sims);
         sims
     }
 }
@@ -290,17 +289,28 @@ impl Approximate<types::Type> for Type {
                 let mut sims: Vec<_> = q
                     .iter()
                     .zip(i.iter())
-                    .map(|(q, i)| q.approx(i, generics, substs))
+                    .filter_map(|(q, i)| q.as_ref().map(|q| q.approx(i, generics, substs)))
                     .flatten()
                     .collect();
 
+                // They are both tuples
+                sims.push(Equivalent);
+
                 // NOTE(hkmatsumoto): Do we actually need to do the same thing when `q.len() > i.len()`?
                 if i.len() > q.len() {
                     sims.append(&mut vec![Different; i.len() - q.len()]);
                 }
                 sims
             }
-            (Slice(q), types::Type::Slice(i)) => q.approx(i, generics, substs),
+            (Slice(q), types::Type::Slice(i)) => {
+                // They are both slices
+                let mut sims = vec![Equivalent];
+
+                if let Some(q) = q {
+                    sims.append(&mut q.approx(i, generics, substs))
+                }
+                sims
+            }
             (Never, types::Type::Never) => vec![Equivalent],
             (
                 RawPointer {
@@ -366,15 +376,19 @@ impl Approximate<types::Type> for Type {
                                 types::GenericArgs::AngleBracketed { args: i, .. },
                             ) = (&**q, &**i)
                             {
-                                let q = q.iter().map(|q| match q {
-                                    GenericArg::Type(q) => q,
+                                let q = q.iter().map(|q| {
+                                    q.as_ref().map(|q| match q {
+                                        GenericArg::Type(q) => q,
+                                    })
                                 });
                                 let i = i.iter().filter_map(|i| match i {
                                     types::GenericArg::Type(t) => Some(t),
                                     _ => None,
                                 });
                                 for (q, i) in q.zip(i) {
-                                    sims.append(&mut q.approx(i, generics, substs))
+                                    if let Some(q) = q {
+                                        sims.append(&mut q.approx(i, generics, substs))
+                                    }
                                 }
                             }
                         }
